// CustomerRegister .java
/** @Author Michael Claudius, Roskilde Computer Science College
 * @Version 1.4. 2010.11.20,
 * Copyright 2010 by Michael Claudius
 * All rights reserved
 */
package Model;

import Model.Customer;
import java.util.*;

public class CustomerRegister {

// Assignment 2
    /** Arraylist holding Person objects */
    private ArrayList<Customer> customers;

// Assignment 2  Declaration of constructors
    /** Default constructor. Initialize the PhoneBook list */
    public CustomerRegister() {
        customers = new ArrayList<Customer>();
    }//constructor

// Assignment 2
    /** Add a Person object to the PhoneBook list. Always return true
    @param aCustomer
    @return boolean true always*/
    public boolean addCustomer(Customer aCustomer) {
        //Add customer to the list
        customers.add(aCustomer);
        return true;
    }

// Assignment 2 building up a string of customers data
    /** Return all friends data as a string
     * Traverse friends list and add each friend's data to a String s
     * @return String whole arraylist friends data as a string
     */
    public String toString() {
        //Traverse and collect customer data to String s
        String s = "";
        for (Customer c : customers) {
            s = s + c.toString() + "\n";
        }
        return s;
    }

    //Alternatively using the toString method in ArrayList
    public String toString1() {
        return customers.toString();
    }

// Assignment 4 Traverse collection. Search and return aCustomer
    /** Return first friend in PhoneBook with a given name.
     * Traverse list and compare names.
     * if the names are equal return the Person object
     * else return null
     * @param customerNo
     * @return Customer found
     */
    public Customer getCustomer(String customerNo) {
        String myCustomerNo;
        for (Customer c : customers) {
            myCustomerNo = c.getCustomerNo();
            if (myCustomerNo.equals(customerNo)) {
                return c;
            }
        }
        return null; // not found

    }

// Assignment 5 
    /**Create a customer, add to list; always return true
     * @param list of customer data
     * @return true as always successfull
     */
    public boolean registerCustomer(String customerNo, String name, String bType, int year) {
        Customer aCustomer = new Customer(customerNo, name, bType, year);
        customers.add(aCustomer);
        return true;
    }

// Assignment 6. 
    /** Removes and returns first person in PhoneBook with a given name.
     * Traverses collection and compares names.
     * @param customerNo
     * @return Customer found
     */
    public Customer removeCustomer(String customerNo) {
        String myCustomerNo;
        for (Customer c : customers) {
            myCustomerNo = c.getCustomerNo();
            if (myCustomerNo.equals(customerNo)) {
                customers.remove(c);
                return c;
            }
        }

        return null; // not found

    }

    // Alternatively
    // Assignment 6. Utilize getCustomer. Typecast and return aCustomer
    public Customer removeCustomerNew(String customerNo) {
        Customer aCustomer = getCustomer(customerNo);
        if (aCustomer != null) {
            //found one to be removed
            customers.remove(aCustomer);
        }
        return aCustomer;
    }

    // Assignment 6. Customer is known. Remove and return aCustomer
    public boolean removeCustomer(Customer b) {
        return customers.remove(b);

    }

//Assignment 7. Note there is no maximum of customers
    public int numberOfCustomers() {
        return customers.size();
    }

//Assignment 8. Find customer with index. Check index. Typecast, Remove and remove aCustomer
    /** Returns a Person object (an instance) at the given index
     * if index is within the boundaries of the arraylist
     * i.e. 0 <= index and index < number of elements
     * else returns null
     * @param index integer of the position in the list
     * @return Person the friend object at position index 
     */
    public Customer getCustomer(int index) {
        if (index >= 0 && index < customers.size()) {
            Customer aCustomer = (Customer) customers.get(index);
            return aCustomer;
        } else {
            return null;
        }
    }

// Assignment 9. Remove customer with index. Check index. Remove aCustomer
    /** Removes a Person object (an instance) at the given index 
     * if index is within the boundaries of the arraylist
     * i.e. 0 <= index and index < number of elements
     * else returns null
     * @param index integer of the position in the list
     * @return Person the friend object which was removed from index position 
     */
    public Customer removeCustomer(int index) {
        if (index >= 0 && index < customers.size()) {
            Customer aCustomer = (Customer) customers.get(index);
            customers.remove(index);
            return aCustomer;
        } else {
            return null;
        }
    }

//Assignment 10. Michaels choice to edit a customer
    public void editCustomer(String customerNo, String newCustomerNo, String name, String bType, int year, String state) {

        for (Customer c : customers) {

            if (c.getCustomerNo().equals(customerNo)) {
                c.setCustomerNo(newCustomerNo);
                c.setName(name);
////                c.setCustomerType(bType);
////                c.setYear(year);
            }
        }
    }

    public boolean editCustomer1(Customer oldCustomer, Customer newCustomer) {
        int index = customers.indexOf(oldCustomer);
        if (index > -1) {
            customers.set(index, newCustomer);
            return true;
        } else {
            return false;
        }
    }

    //Assignment 10. Michaels choice. Replace a customer with a customerno, if any
    public Customer replaceCustomer(String customerNo, Customer customer) {
        Customer aCustomer;
        String myCustomerNo;
        for (int index = 0; index < customers.size(); index++) {
            aCustomer = (Customer) customers.get(index);
            myCustomerNo = aCustomer.getCustomerNo();
            if (myCustomerNo.equals(customerNo)) {
                customers.set(index, customer);
                return aCustomer; // assumes that its the old one which is returned
            }
        }

        return null; // not found
    }

// Assignment 10. Insert customer at index
    /** Insert a Person object at index position
     * returns true if index is within the boundaries of the arraylist
     * else returns false
     * @param aFriend the person object to be inserted
     * @param index integer of the position in the list
     * @return boolean true if inserted else false
     */
    public boolean insertCustomer(Customer customer, int index) {
        if (index >= 0 && index < customers.size()) {
            customers.add(index, customer);
            return true;
        } else {
            return false;
        }
    }

// Assignment X. Remove all customers with the same name. 
//@Returns number of removed customers
    public int removeYoungFriends(int year) {
        Iterator<Customer> it = customers.iterator();
        int total = 0;
        while (it.hasNext()) {
            Customer c = it.next();
            if (c.getYear() < year) {
                it.remove();
                total++;
            }
        }
        return total;
    }

// Assignment Y Remove all customers with the same name
    public CustomerRegister removeCustomersByName(String name) {
        Customer aCustomer;
        String myName;
        CustomerRegister foundCustomers = new CustomerRegister();
        for (int index = 0; index < customers.size(); index++) {
            aCustomer = (Customer) customers.get(index);
            myName = aCustomer.getName();
            if (myName.equals(name)) {
                foundCustomers.addCustomer(aCustomer);
                customers.remove(index);
                index--;
            }
        }
        return foundCustomers;
    }

// Assignment Y Find all customers with the same name
    public CustomerRegister getCustomersByName(String name) {
        Customer aCustomer;
        String myName;
        CustomerRegister foundCustomers = new CustomerRegister();
        for (Customer customer : customers) {
            myName = customer.getName();
            if (myName.equals(name)) {
                foundCustomers.addCustomer(customer);
            }
        }
        return foundCustomers;
    }
}
